/*
 * File:   mutualfund.h
 */

#ifndef MUTUALFUND_H
#define MUTUALFUND_H

#include <string>
#include <set>
#include <map>
#include "Date.h"

const string Path = "c:/temp/";

class Dividend
{
    Date date;
    float div; // amount / share
public:
    Dividend(const Date&, float = 0);

    Date get_date() const
    {
        return date;
    }

    float get_div() const
    {
        return div;
    }

    bool operator<(const Dividend& obj) const
    {
        return date < obj.date;
    }
};

std::ostream& operator<<(ostream& out, const Dividend& obj);

class MutualFund
{
    std::string ticker;
    Date oldestDate;
    Date newestDate;
    float newestNav;
    std::map<Date, float> navHistory;
    std::set<Dividend> divHistory;
    void readMutualFundHistoryFile(const std::string& filename);
    void readMutualFundDividendFile(const std::string& filename);
    Date getNthDateFromNavHistory(int n) const;
    std::map<Date, float>::const_iterator getNthNavFromNavHistory(int n) const;
    void writeNavDataToFile() const;
    void writeDivDataToFile() const;

    const std::set<Dividend>& get_divHistory() const
    {
        return divHistory;
    }
    float getMovingAverage(int NumDays) const;
    float ThreeYearGain() const;
    pair<float, float> getMinMaxForTheLastYear() const;

    Date get_oldestDate() const
    {
        return oldestDate;
    }

    Date get_newestDate() const
    {
        return newestDate;
    }

    float get_newestNav() const
    {
        return newestNav;
    }

public:
    MutualFund() = default;
    MutualFund(const std::string& tick);
    void report() const;
    std::pair<std::map<Date, float>::const_iterator, std::map<Date, float>::const_iterator> getTwoRandomNavs() const;

    std::string getTicker() const
    {
        return ticker;
    }
    float getReturnOverTime(float initialInvestment,const Date& start, const Date& end) const;
    float applyDividends(float sharesOwned,const Date& start,const Date& end) const;
    static constexpr float InitialInvestment = 10000.f;
};

ostream& operator<<(ostream& out, const std::pair<Date, float>& obj);

ostream& operator<<(ostream& out, const std::map<Date, float>& obj);

size_t locationOfNthCommaInString(const std::string& s, unsigned short whichOne);

bool navCompare(const pair<Date, float>& a, const pair<Date, float>& b);
#endif /* MUTUALFUND_H */

